import * as log from './log'
import * as events from 'events'

export type ChainItemFunc = (message: any) => any

export interface Receivable {
    begin() : void;
    send(message : any) : void;
}

class LastChainItem implements Receivable {
    public begin() : void {
    }

    public send(message: any) {
        log.info(`Last chain item, final result ${message}`)
    }
}

let lastChainItem : LastChainItem = new LastChainItem();

export abstract class AChainItem extends events.EventEmitter implements Receivable {
    protected _label : string
    protected _next : Receivable


    constructor(label : string) {
        super()
        this._label = label
        this._next = lastChainItem
    }

    public toString() : string {
        return `<${this._label}>`
    }

    public registerNext(next : AChainItem) :void {
        this._next = next
    }

    public send(message : any) : void {
        this.emit('yourturn', message)
    }

    public abstract begin() : void
}

export abstract class ABaseChainItem extends AChainItem  {
    constructor(label : string) {
        super(label)
    }

    protected abstract func(message : any) : any;

    public begin() :void {
        if (this._next != null) {
            this._next.begin()
        }
        this.addListener('yourturn', (message : any) : void => {
            log.info(`Calling ${this.toString()}`)
            let result : any = this.func(message)
            this._next.send(result)
        })
    }
}

export class ChainItem extends ABaseChainItem  {
    private _chainItemFunc : ChainItemFunc

    public toString() : string {
        let funcName : string = (this._chainItemFunc.name === "") ? "lambda" : this._chainItemFunc.name
        return `<${funcName}, ${this._label}>`
    }

    public registerFunc(func : ChainItemFunc) : void {
        this._chainItemFunc = func
    }

    protected func(message : any) : any {
        return this._chainItemFunc(message)
    }
}

