import * as CallChain from './callchain'
import * as log from './log'
import * as MongoDB from 'mongodb'


export class MongoSave extends CallChain.ABaseChainItem  {
    private _url : string
    private _mongoClient : MongoDB.MongoClient
    private _dbh : MongoDB.Db | undefined
    private _connectPending : boolean

    constructor(url:string) {
        super('MongoSave')
        this._url = url
        this._mongoClient = new MongoDB.MongoClient()
        this._connectPending = false
    }

    protected func(message : any) : any {
        if (! this._dbh) {
            log.info("Not database connection yet")
            if (! this._connectPending) {
                this._connectPending = true
                this._mongoClient.connect(this._url)
                    .then((db:MongoDB.Db) => { 
                        log.info("Successfully opened MongoDB connect")
                        this._dbh = db
                    })
                    .catch((err) => {
                        log.error(`Failure when opening MongoDB connect: ${err}`)
                        this._dbh = undefined
                    })
            } else {
                log.info("Connecting to database is pending")
            }
        }
        if (this._dbh) {
            log.info("Database handle is available")
            let coll : MongoDB.Collection = this._dbh.collection("iot")
            coll.insertOne(message)
                .then((res : MongoDB.InsertOneWriteOpResult) => {
                    log.info(`Successfully wrote one item in database: ${res.insertedId}`)
                })
                .catch((err : any) => {
                    log.error(`Failure when trying to write one item in database: ${err}`)
                })
        } else {
            log.error(`No database connection yet, drop message ${message}`)
        }
        return "<<" + message + ">>"
    }

    /*
    public begin() :void {
        if (this._next != null) {
            this._next.begin()
        }
        this.addListener('yourturn', (message : any) : void => {
            log.info(`Calling ${this.toString()}`)
            let result : any = this.func(message)
            if (this._next == null) {
                log.info(`Last chain item, final result ${result}`)
            } else {
                this._next.send(result)
            }
        })
    }
    */

}
